# frozen_string_literal: true

require 'httparty'
require_relative 'logger'
require_relative 'response_handler'

module Minato
  module Utils
    class Client
      include HTTParty

      def initialize(base_uri:, default_options: {})
        @base_uri = base_uri
        @default_options = default_options
      end

      def get(path, options = {})
        request(:get, path, options)
      end

      def post(path, options = {})
        request(:post, path, options)
      end

      def put(path, options = {})
        request(:put, path, options)
      end

      def patch(path, options = {})
        request(:patch, path, options)
      end

      def delete(path, options = {})
        request(:delete, path, options)
      end

      private

      def request(method, path, options)
        request_options = @default_options.deep_merge(options)
        url = Minato::Utils::Helpers.join_url(@base_uri, path)

        Minato::Utils.logger.log_request(method, @base_uri, path, request_options)
        response = self.class.send(method, url, request_options)

        Minato::Utils.logger.log_response(method, @base_uri, path, request_options, response)
        Minato::Utils::ResponseHandler.handle(response)
      end
    end
  end
end
