# frozen_string_literal: true

module Minato
  module Utils
    class SerializableError < StandardError
      attr_reader :id, :code, :body

      def initialize(object)
        @id = self.class.to_s.underscore
        @body = object.body
        @code = object.code
        super
      end
    end

    class SerializableException < SerializableError; end

    class NetworkException < SerializableError
      def initialize(response)
        super(response)
        @body = Minato::Utils::Helpers.parse_json(response.body)
      end
    end

    class BadGateway < NetworkException; end

    class BadRequest < NetworkException; end

    class Conflict < NetworkException; end

    class Forbidden < NetworkException; end

    class Gone < NetworkException; end

    class NotFound < NetworkException; end

    class ServerError < NetworkException; end

    class ServiceUnavailable < NetworkException; end

    class Unauthorized < NetworkException; end

    class UnprocessableEntity < NetworkException; end
  end
end
