# frozen_string_literal: true

require 'active_support'
require 'active_support/core_ext/enumerable'

module Minato
  module Utils
    class ResponseHandler
      class << self
        def handle(response)
          raise_exception(response) if response_error?(response)

          body = Minato::Utils::Helpers.parse_json(response.body)
          header = response.header.to_hash || {}
          { body: body, header: header }
        end

        private

        SUCCESSFUL_RESPONSE_CODE = [200, 201, 204, 302, 303].freeze

        EXCEPTION_HANDLERS_MAP = {
          400 => 'bad_request',
          401 => 'unauthorized',
          403 => 'forbidden',
          404 => 'not_found',
          409 => 'conflict',
          410 => 'gone',
          422 => 'unprocessable_entity',
          500 => 'server_error',
          502 => 'bad_gateway',
          503 => 'service_unavailable'
        }.freeze

        def response_error?(response)
          SUCCESSFUL_RESPONSE_CODE.exclude?(response.code)
        end

        def raise_exception(response)
          raise get_exception_class_from(response.code), response
        end

        def get_exception_class_from(code)
          "Minato::Utils::#{EXCEPTION_HANDLERS_MAP[code].camelize}".constantize
        end
      end
    end
  end
end
